@@ -13,12 +13,13 @@ import urllib
 import keyword
 import warnings
 import traceback
+import unicodedata
 
 import genshi.builder as bldr
 from genshi.core import Stream, Markup
 
 from core import (escape_char, esc_neg_look, fragmentize,
-                  ImplicitList, AttrDict) 
+                  ImplicitList, AttrDict, MacroError) 
 
 BLOCK_ONLY_TAGS = ['h1','h2','h3','h4','h5','h6',
               'ul','ol','dl',
@@ -28,8 +29,7 @@ BLOCK_ONLY_TAGS = ['h1','h2','h3','h4','h5','h6',
 
 BLOCK_TAGS = BLOCK_ONLY_TAGS + ['ins','del','script']
 
-
-MACRO_NAME = r'(?P<name>[a-zA-Z]+([-.]?[a-zA-Z0-9]+)*)'
+MACRO_NAME = r'(?P<name>[a-zA-Z][a-zA-Z0-9]*([-.][a-zA-Z0-9]+)*)'
 """allows any number of non-repeating hyphens or periods.
 Underscore is not included because hyphen is"""
 
@@ -281,7 +281,7 @@ class LinkElement(InlineElement):
     def __init__(self,tag, token, delimiter,
                  interwiki_delimiter,base_urls,links_funcs,
                  interwiki_class_funcs,default_space_char,space_chars,
-                 base_url,space_char,class_func,path_func):
+                 base_url,space_char,class_func,path_func,fragment_pattern):
         super(LinkElement,self).__init__(tag,token)
         self.regexp = re.compile(self.re_string(),re.DOTALL)
         self.delimiter = delimiter
@@ -295,6 +295,7 @@ class LinkElement(InlineElement):
         self.space_char = space_char
         self.class_func = class_func
         self.path_func = path_func
+        self.fragment_pattern = fragment_pattern
         self.content_regexp = re.compile(self.content_re_string(),re.DOTALL)
         self.interwikilink_regexp = re.compile(self.interwikilink_re_string())
         self.urllink_regexp = re.compile(self.urllink_re_string(), re.DOTALL)
@@ -319,7 +320,11 @@ class LinkElement(InlineElement):
 
     def wikilink_re_string(self):
         optional_spaces = ' *'
-        page_name = r'(?P<page_name>\S+?( \S+?)*?)' #allows any number of single spaces
+        if self.fragment_pattern:
+            page_name = r'(?P<page_name>\S*?( \S+?)*?)(?P<fragment>' + \
+                        self.fragment_pattern + ')?'
+        else:
+            page_name = r'(?P<page_name>\S+?( \S+?)*?)' #allows any number of single spaces
         return '^' + optional_spaces + page_name + optional_spaces + '$'
 
     def page_name(self,mo):
@@ -360,10 +365,12 @@ class LinkElement(InlineElement):
             wikilink_mo = self.wikilink_regexp.match(body)
             link_type = 'wiki'
             page_name = self.page_name(wikilink_mo)
-            if self.path_func:
+            if self.path_func and page_name:
                 the_path = self.path_func(page_name)
             else:
                 the_path = urllib.quote(page_name.encode('utf-8'))
+            if wikilink_mo.groupdict().get('fragment'):
+                the_path = ''.join([the_path,wikilink_mo.group('fragment')])
             if self.class_func:
                 the_class = self.class_func(page_name)
             url = urlparse.urljoin(self.base_url, the_path)
@@ -394,7 +401,8 @@ class AnchorElement(LinkElement):
     ...                bigwiki='http://bigwiki.net/'),
     ... links_funcs={},interwiki_class_funcs={},default_space_char='-',
     ... space_chars={'bigwiki':' '},base_url='http://somewiki.org/',
-    ... space_char='_',class_func=None,path_func=None,external_links_class=None)
+    ... space_char='_',class_func=None,path_func=None,external_links_class=None,
+    ... fragment_pattern=r'#![a-z0-9-_]+')
     
     >>> mo = link.regexp.search("[[http://www.google.com| here]]")
     >>> link._build(mo,{},None).generate().render()
@@ -411,6 +419,14 @@ class AnchorElement(LinkElement):
     >>> mo = link.regexp.search(" [[Home Page |Home]]")
     >>> link._build(mo,{},None).generate().render()
     '<a href="http://somewiki.org/Home_Page">Home</a>'
+
+    >>> mo = link.regexp.search(" [[#!my-header|Subpage]]")
+    >>> link._build(mo,{},None).generate().render()
+    '<a href="http://somewiki.org/#!my-header">Subpage</a>'
+
+    >>> mo = link.regexp.search(" [[Home Page#!a-header |Home]]")
+    >>> link._build(mo,{},None).generate().render()
+    '<a href="http://somewiki.org/Home_Page#!a-header">Home</a>'
     
     """
 
@@ -441,7 +457,8 @@ class ImageElement(LinkElement):
     ...                bigwiki='http://bigwiki.net/'),
     ... links_funcs={},interwiki_class_funcs={},default_space_char='-',
     ... space_chars={'bigwiki':' '},base_url='/images/',
-    ... space_char='_',class_func=None,path_func=None,disable_external=False)
+    ... space_char='_',class_func=None,path_func=None,disable_external=False,
+    ... fragment_pattern=False)
 
     >>> mo = img.regexp.search('{{ picture.jpg | An image of a house }}')
     >>> img._build(mo,{},None).generate().render()
@@ -578,6 +595,9 @@ class Macro(WikiElement):
                       + re.sub('arguments?','argument(s)',mo.group(3)) \
                       + str(int(mo.group(4))-2) + ' given)'
             value = bldr.tag.__getattr__(tag)("Macro error: '%s' %s"% (macro_name, msg),class_='macro_error')
+        except MacroError, detail:
+            tag = isblock and 'pre' or 'code'
+            value = bldr.tag.__getattr__(tag)("Error in macro '%s': %s"% (macro_name,str(detail.value)),class_='macro_error')
         except:
             value = bldr.tag.pre('Unexpected error during macro execution!\n'
                                  + traceback.format_exc(20) ,
@@ -1171,22 +1191,58 @@ class Paragraph(BlockElement):
 
 class Heading(BlockElement):
 
-    r"""Finds heading wiki elements.
+    r"""Processes heading wiki elements. Optionally adds id attributes.
 
-    >>> h1 = Heading(['h1','h2'],'=')
+    >>> h1 = Heading(['h1','h2'],'=',id_prefix=False)
     >>> mo = h1.regexp.search('before\n = An important thing = \n after')
     >>> mo.group(2)
     'An important thing'
     >>> mo.group(0)
     ' = An important thing = \n'
 
+    >>> h1 = Heading(['h1','h2'],'=',id_prefix='!')
+    >>> mo = h1.regexp.search(u'before\n = An important thing = \n after')
+    >>> mo.group(2)
+    u'An important thing'
+    >>> h1.make_id('!',mo.group(2),[])
+    '!an-important-thing'
+    >>> h1.make_id('!',mo.group(2),['!an-important-thing'])
+    '!an-important-thing_1'
+    >>> h1.make_id('!',mo.group(2),{})
+    '!an-important-thing'
+    >>> h1.make_id('!',mo.group(2) + '!@#$%&*()_[]+=/?|-- TO    SAY',{})
+    '!an-important-thing-to-say'
+
     """
-  
-    def __init__(self, tag, token):
+
+    def __init__(self, tag, token, id_prefix):
         super(Heading,self).__init__('',token)
+        self.id_prefix = id_prefix
         self.tags = tag
         self.regexp = re.compile(self.re_string(),re.MULTILINE)
 
+
+    def make_id(self, prefix, heading_text,used_ids):
+        slug = unicodedata.normalize('NFKD', heading_text).encode('ascii', 'ignore').lower()
+        slug = re.sub('-+', '-', re.sub('[^a-z0-9-]+', '-', slug)).strip('-')
+        slug = ''.join([prefix,slug]) or '!'
+        # Restrict length without breaking words.
+        while len(slug) > 40:
+            if slug.find('-') == -1:
+                slug = slug[:40]
+            else:
+                slug = slug.rsplit('-', 1)[0]
+        i = 1
+        original_slug = slug 
+        while True:
+            if slug not in used_ids:
+                break
+            slug = '%s_%d' % (original_slug, i)
+            i += 1
+        
+        return slug
+
+  
     def re_string(self):
         whitespace = r'[ \t]*'
         tokens = '(' + re.escape(self.token) + '{1,' + str(len(self.tags)) +'})'
@@ -1197,9 +1253,22 @@ class Heading(BlockElement):
 
     def _build(self,mo,element_store, environ):
         heading_tag = self.tags[len(mo.group(1))-1]
-        return bldr.tag.__getattr__(heading_tag)(fragmentize(mo.group(2),
-                                                          self.child_elements,
-                                                          element_store, environ))
+        heading_body = fragmentize(mo.group(2),
+                                   self.child_elements,
+                                   element_store, environ)
+        if self.id_prefix is False:
+            id_=None
+        else:
+            heading_text = bldr.tag(heading_body).generate().render(method='text',encoding=None)
+            used_ids = environ.setdefault('ids', [])
+            id_ = self.make_id(self.id_prefix,heading_text,used_ids)
+            used_ids.append(id_)
+            #toc = environ.setdefault('toc', [])
+            #toc.append((heading_tag, bldr.tag(heading_body), id_))            
+        return bldr.tag.__getattr__(heading_tag)(heading_body,
+                                                 id_=id_)
+
+
 
 
 class Table(BlockElement):
